package com.vendor.burn.model.devices.burners;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.model.AlgorithmType;
import com.vendor.burn.model.BurnContext;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.FlashType;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.TransferWay;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.define.IFastbootProgressMonitor;
import com.vendor.burn.model.devices.ICheckSumBurner;
import com.vendor.burn.model.devices.ITab;
import com.vendor.burn.model.devices.burners.PartitionBurner;
import com.vendor.burn.model.event.EventType;
import com.vendor.burn.model.event.FastbootEvent;
import com.vendor.burn.serial.workers.AbstractBurnerWorker;
import com.vendor.burn.serial.workers.CheckSumBurnerWorker;
import com.vendor.burn.serial.workers.PartitionBurnerWorker;
import com.vendor.burn.ui.burners.tabs.CheckSumTab;
import com.vendor.burn.utilitily.ToolUtils;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeListener;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;

public class CheckSumBurner extends PartitionBurner implements ICheckSumBurner {
  public CheckSumBurner(BurnContext burnContext) {
    super(burnContext);
    getChildren().clear();
  }
  
  public void checkSum(IFastbootProgressMonitor fastbootMonitor, File file, Text checkSumResultText, AlgorithmType algorithmType) {
    CheckSumBurnerWorker worker = getBurnerWorker();
    worker.setProgressMonitor(fastbootMonitor);
    FastbootEvent event = fastbootMonitor.getEvent();
    Object object = new Object(this, "Checking Sum...", worker, file, checkSumResultText, algorithmType, event, fastbootMonitor);
    showProgressDialog((Job)object);
    object.addJobChangeListener((IJobChangeListener)new Object(this, fastbootMonitor));
    object.schedule();
  }
  
  public List<String> getSteps() {
    List<String> steps = new ArrayList<>();
    steps.add(Messages.MergeBurner_Step_1);
    steps.add(Messages.MergeBurner_Step_2);
    steps.add(Messages.MergeBurner_Step_3);
    return steps;
  }
  
  public CheckSumBurnerWorker getBurnerWorker() {
    return new CheckSumBurnerWorker((IBurner)this);
  }
  
  public IStatus checkBeforeBurn() {
    IStatus check = checkTable(EventType.BURN);
    if (!isOK(check))
      return check; 
    return null;
  }
  
  public ITab getTab() {
    return (ITab)new CheckSumTab();
  }
  
  protected IStatus checkTable(EventType event) {
    List<PartitionItem> items = getChildren();
    int size = items.size();
    int index = 0;
    int selection = 0;
    for (index = 0; index < size; index++) {
      if (((PartitionItem)items.get(index)).isChecked()) {
        selection++;
        break;
      } 
    } 
    if (0 == selection)
      return error(Messages.PartitionBurner_msg_no_burning_partition); 
    IStatus check = checkOtherPart(items, event);
    if (!isOK(check))
      return check; 
    return null;
  }
  
  protected IStatus checkOtherPart(List<PartitionItem> parts, EventType event) {
    int size = parts.size();
    int index = 0;
    long spiTotalLength = 0L;
    long nandTotalLength = 0L;
    long emmcTotalLength = 0L;
    boolean isContinuous = false;
    for (index = 0; index < size - 1; index++) {
      long thisStart = ((PartitionItem)parts.get(index)).getStart();
      long thisLen = ((PartitionItem)parts.get(index)).getLength();
      long nextStart = ((PartitionItem)parts.get(index + 1)).getStart();
      if (thisStart + thisLen < nextStart) {
        highLight(parts, index);
        isContinuous = true;
      } 
    } 
    if (isContinuous) {
      boolean isConfirm = MessageDialog.openConfirm(Display.getDefault()
          .getActiveShell(), Messages.FASTBOOT_WARNING_Title, Messages.MergeComposite_MergeImage_Must_Connection);
      if (!isConfirm)
        return Status.CANCEL_STATUS; 
    } 
    FileSystem[] fileSystems = FileSystem.getFileSystems(this);
    IStatus result = Status.OK_STATUS;
    for (index = 0; index < size; index++) {
      PartitionItem part = parts.get(index);
      FileSystem fileSystem = part.getFileSystem();
      FlashType flashType = part.getFlashType();
      if (part.isChecked()) {
        long start = part.getStart();
        long length = part.getLength();
        result = checkAddressOverlap(parts, part, flashType, spiTotalLength, nandTotalLength, emmcTotalLength, index);
        if (result != Status.OK_STATUS)
          return result; 
        result = checkFileType(index, fileSystems, part, fileSystem);
        if (result != Status.OK_STATUS)
          return result; 
        boolean checkLen = (index != size - 1 && 0L == length);
        if (checkLen) {
          highLight(part);
          return error(
              NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_nonzero_in_row_, 
                Integer.valueOf(index + 1)));
        } 
        File file = part.getFile();
        if (file != null) {
          boolean fileInvild = (!file.exists() || file.isDirectory());
          if (fileInvild)
            return error(
                NLS.bind(Messages.AbstractBurner_msg_file_does_not_exist_in_row_, 
                  Integer.valueOf(index + 1))); 
          long fileSize = file.length();
          if (fileSize == 0L) {
            highLight(part);
            return error(
                NLS.bind(Messages.AbstractBurner_msg_the_file_size_must_be_nonzero_in_row_, 
                  Integer.valueOf(index + 1)));
          } 
          if (index != size - 1) {
            if (length < fileSize) {
              highLight(part);
              return error(
                  NLS.bind(Messages.PartitionBurner_msg_invalid_partition_size_in_row_, part
                    .getFile().getName()));
            } 
            if (0L == length) {
              highLight(part);
              return error(
                  NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_nonzero_in_row_, 
                    Integer.valueOf(index + 1)));
            } 
            if (length < 0L) {
              highLight(part);
              return error(
                  NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_Less_than_zero_in_row_, 
                    Integer.valueOf(index + 1)));
            } 
            if (start < 0L) {
              highLight(part);
              return error(
                  NLS.bind(Messages.AbstractBurner_msg_the_partition_address_must_be_Less_than_zero_in_row_, 
                    Integer.valueOf(index + 1)));
            } 
          } else {
            if (length < -1L) {
              highLight(part);
              return error(
                  NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_Less_than_zero_in_row_, 
                    Integer.valueOf(index + 1)));
            } 
            if (start < 0L) {
              highLight(part);
              return error(
                  NLS.bind(Messages.AbstractBurner_msg_the_partition_address_must_be_Less_than_zero_in_row_, 
                    Integer.valueOf(index + 1)));
            } 
          } 
          boolean yaffsFile = false;
          try {
            yaffsFile = FileSystem.isYaffsFile((IBurner)this, file);
          } catch (IOException e) {
            highLight(part);
            return error(
                NLS.bind(Messages.PartitionBurner_msg_failed_to_open_the_file_in_row_, 
                  Integer.valueOf(index + 1)));
          } 
          boolean isVaildYaffs = (yaffsFile && FileSystem.yaffs != fileSystem);
          if (isVaildYaffs) {
            highLight(part);
            return error(
                NLS.bind(Messages.PartitionBurner_msg_file_is_yaffs_but_file_system_is_not_in_row_, 
                  Integer.valueOf(index + 1)));
          } 
          int alignLen = 512;
          if (flashType == FlashType.ufs)
            alignLen = 4096; 
          if (start % alignLen != 0L) {
            highLight(part);
            return error(
                NLS.bind(Messages.PartitionBurner_msg_partition_Start_should_be_with_unit_KorM_or_NK_times, 
                  Integer.valueOf(alignLen), Integer.valueOf(index + 1)));
          } 
          boolean needAlign = (index != size - 1 && length % alignLen != 0L);
          if (needAlign) {
            highLight(part);
            return error(
                NLS.bind(Messages.PartitionBurner_msg_Length_must_be_NK_times_that_is_should_be_with_unit_K_or_M_in_row_, 
                  Integer.valueOf(alignLen), Integer.valueOf(index + 1)));
          } 
          try {
            boolean checkTftpFileNameInvaild = ((getBurnContext().getTransferWay() == TransferWay.Net || getBurnContext().getTransferWay() == TransferWay.UsbTransNet || getBurnContext().getTransferWay() == TransferWay.JTAGAndNet) && (file.getName().getBytes(
                System.getProperty("file.encoding"))).length > 100);
            if (checkTftpFileNameInvaild)
              return error(
                  NLS.bind(Messages.msg_File_name_is_too_long_please_use_a_short_name_instead, 
                    Integer.valueOf(index + 1))); 
          } catch (UnsupportedEncodingException unsupportedEncodingException) {}
        } 
      } 
    } 
    return Status.OK_STATUS;
  }
  
  private void highLight(List<PartitionItem> parts, int index) {
    if (!ToolUtils.isCmdLine())
      getBurnContext().getUIAdapter().highLight(parts.get(index)); 
  }
  
  private IStatus checkFileType(int index, FileSystem[] fileSystems, PartitionItem part, FileSystem fileSystem) {
    boolean validFileType = false;
    for (FileSystem req : fileSystems) {
      if (req == fileSystem) {
        validFileType = true;
        break;
      } 
    } 
    if (!validFileType) {
      highLight(part);
      return error(NLS.bind(Messages.AbstractBurner_msg_invalid_file_system_in_row_, 
            
            Integer.valueOf(index + 1)));
    } 
    return Status.OK_STATUS;
  }
  
  protected IStatus checkAddressOverlap(List<PartitionItem> parts, PartitionItem part, FlashType flashType, long spiTotalLength, long nandTotalLength, long emmcTotalLength, int index) {
    long start = part.getStart();
    PartitionItem lastPart = null;
    boolean isSelectNextPart = false;
    FlashType lastFlashType = null;
    long lastStart = 0L;
    long lastLength = 0L;
    long lastTotalLength = 0L;
    if (index - 1 >= 0) {
      lastPart = parts.get(index - 1);
      lastFlashType = lastPart.getFlashType();
      if (lastPart.isChecked() && flashType == lastFlashType) {
        isSelectNextPart = true;
        lastStart = lastPart.getStart();
        lastLength = lastPart.getLength();
        lastTotalLength = lastStart + lastLength;
      } 
    } 
    if (FlashType.spi == flashType) {
      if (0L == start && 0L != spiTotalLength) {
        highLight(part);
        return error(
            NLS.bind(Messages.AbstractBurner_msg_the_partition_address_must_be_nonzero_in_row_, 
              Integer.valueOf(index + 1)));
      } 
      if (0L != spiTotalLength && start < spiTotalLength) {
        highLight(part);
        return error(NLS.bind(Messages.PartitionBurner_msg_address_overlap_in_row_, 
              
              Integer.valueOf(index + 1)));
      } 
      if (isSelectNextPart && start < lastTotalLength) {
        highLight(part);
        return error(NLS.bind(Messages.PartitionBurner_msg_address_overlap_in_row_, 
              
              Integer.valueOf(index + 1)));
      } 
    } else if (FlashType.nand == flashType) {
      if (0L == start && 0L != nandTotalLength) {
        highLight(part);
        return error(
            NLS.bind(Messages.AbstractBurner_msg_the_partition_address_must_be_nonzero_in_row_, 
              Integer.valueOf(index + 1)));
      } 
      if (0L != nandTotalLength && start < nandTotalLength) {
        highLight(part);
        return error(NLS.bind(Messages.PartitionBurner_msg_address_overlap_in_row_, 
              
              Integer.valueOf(index + 1)));
      } 
      if (isSelectNextPart && start < lastTotalLength) {
        highLight(part);
        return error(NLS.bind(Messages.PartitionBurner_msg_address_overlap_in_row_, 
              
              Integer.valueOf(index + 1)));
      } 
    } else if (FlashType.emmc == flashType) {
      if (0L == start && 0L != emmcTotalLength) {
        highLight(part);
        return error(
            NLS.bind(Messages.AbstractBurner_msg_the_partition_address_must_be_nonzero_in_row_, 
              Integer.valueOf(index + 1)));
      } 
      if (isSelectNextPart && start < lastTotalLength) {
        highLight(part);
        return error(NLS.bind(Messages.PartitionBurner_msg_address_overlap_in_row_, 
              
              Integer.valueOf(index + 1)));
      } 
    } else if (FlashType.eeprom != flashType) {
      highLight(part);
      return error(NLS.bind(Messages.AbstractBurner_msg_invalid_flash_type_in_row_, 
            
            Integer.valueOf(index + 1)));
    } 
    return Status.OK_STATUS;
  }
  
  protected void highLight(PartitionItem part) {
    if (!ToolUtils.isCmdLine())
      getBurnContext().getUIAdapter().highLight(part); 
  }
}
